'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

const fs = require('fs');
const sitemap = require('sitemap');
const format = require('xml-formatter');
const utils = require('@antfu/utils');
const path = require('path');
const fg = require('fast-glob');

function resolveOptions(userOptions) {
  return Object.assign(
    {
      hostname: "http://localhost/",
      dynamicRoutes: [],
      exclude: [],
      basePath: "",
      outDir: "dist",
      extensions: "html",
      changefreq: "daily",
      priority: 1,
      lastmod: new Date(),
      readable: false,
      robots: [{
        userAgent: "*",
        allow: "/"
      }]
    },
    userOptions
  );
}

var RobotCorrespondences = /* @__PURE__ */ ((RobotCorrespondences2) => {
  RobotCorrespondences2["userAgent"] = "User-agent";
  RobotCorrespondences2["allow"] = "Allow";
  RobotCorrespondences2["disallow"] = "Disallow";
  RobotCorrespondences2["crawlDelay"] = "Crawl-delay";
  RobotCorrespondences2["cleanParam"] = "Clean-param";
  return RobotCorrespondences2;
})(RobotCorrespondences || {});
function getRules(options) {
  const rules = [];
  options.forEach((rule) => {
    const keys = Object.keys(RobotCorrespondences).filter((key) => typeof rule[key] !== "undefined");
    keys.forEach((key) => {
      const values = Array.isArray(rule[key]) ? rule[key] : [rule[key]];
      values.forEach((value) => {
        rules.push({
          key: RobotCorrespondences[key],
          value
        });
      });
    });
  });
  return rules;
}
function getContent(rules, hostname) {
  return rules.map((rule) => `${rule.key}: ${String(rule.value).trim()}`).join("\n").concat(`

Sitemap: ${getFinalSitemapPath(hostname)}`);
}
function getFinalSitemapPath(hostname) {
  return `${utils.ensureSuffix("/", hostname)}sitemap.xml`;
}

function getResolvedPath(file, resolvedOptions) {
  if (path.isAbsolute(resolvedOptions.outDir))
    return path.resolve(`${resolvedOptions.outDir}/${file}`);
  return path.resolve(`${utils.ensurePrefix("./", resolvedOptions.outDir)}/${file}`);
}
function removeMaybeSuffix(suffix, str) {
  if (!str.endsWith(suffix))
    return str;
  return str.slice(0, -suffix.length);
}

function getRoutes(options) {
  const ext = typeof options.extensions === "string" ? [options.extensions] : options.extensions;
  const strExt = ext.map((e) => `**/*.${e}`);
  return [
    ...fg.sync(strExt, { cwd: options.outDir }).map((route) => {
      let r = route;
      ext.forEach((e) => {
        const regex = new RegExp(`index.${e}`, "g");
        r = r.replace(regex, "");
      });
      const parsedRoute = path.parse(r);
      return utils.slash(path.join("/", parsedRoute.dir, parsedRoute.name));
    }),
    ...options.dynamicRoutes.map((route) => utils.slash(path.join("/", path.join(path.parse(route).dir, path.parse(route).name))))
  ].filter((route) => !options.exclude.includes(route));
}
function getFormattedSitemap(options, routes) {
  return routes.map((route) => ({
    url: new URL(options.basePath ? utils.ensurePrefix("/", options.basePath) + utils.ensurePrefix("/", route) : utils.ensurePrefix("/", "/route"), removeMaybeSuffix("/", options.hostname)).href,
    changefreq: options.changefreq,
    priority: options.priority,
    lastmod: options.lastmod
  }));
}

function generateSitemap(options = {}) {
  const resolvedOptions = resolveOptions(options);
  const robotRules = getRules(resolvedOptions.robots);
  const robotContent = getContent(robotRules, resolvedOptions.hostname);
  fs.writeFileSync(getResolvedPath("robots.txt", resolvedOptions), robotContent);
  const routes = getRoutes(resolvedOptions);
  if (!routes.length)
    return;
  const formattedSitemap = getFormattedSitemap(resolvedOptions, routes);
  const stream = new sitemap.SitemapStream();
  formattedSitemap.forEach((item) => stream.write(item));
  sitemap.streamToPromise(stream).then((sitemap) => {
    const utfSitemap = sitemap.toString("utf-8");
    const formattedSitemap2 = resolvedOptions.readable ? format(utfSitemap) : utfSitemap;
    fs.writeFileSync(getResolvedPath("sitemap.xml", resolvedOptions), formattedSitemap2);
  });
  stream.end();
}

exports.default = generateSitemap;
exports.generateSitemap = generateSitemap;
exports.getFormattedSitemap = getFormattedSitemap;
exports.getRoutes = getRoutes;
exports.resolveOptions = resolveOptions;
